C     Driver test code applying S, D operators via FMM and Alpert corrections
c     onto a smooth closed curve in 2D. Tests interior Green's Representation
c     Theorem (GRF) with jump relations, on the inside limit of curve.
c     Note that when qpnei>0, the extra copies add zero extra field by Calderon
c     (they are orthog to exterior projector), so serves to test them.
c
c     Part of lp2d tools
c
c     Alex Barnett 3/15/11, added check QP-nei-copies-summing 3/24/11
      program lpevalselfcc_testgrf
      implicit none
      integer n, nmax, ier, i, nosc, iprec, alpertord, qpnei
      parameter (nmax=1e5)
      real *8 aosc, pi, t, node(2,nmax), nvec(2,nmax), sp(nmax), r
      real *8 xp, yp, rp, src(2), snor(2), dummy, errpot, t1, t2
      real *8 qpdvec(2)
      complex *16 ima, zk, sig(nmax), tau(nmax), pot(nmax), nd(nmax)
      complex *16 charge, f(4), qpalpha
      parameter (pi = 3.141592653589793d0, ima = (0, 1))
      call prini(6,0)

c     # quadr points, and wavenumber...
      n = 1e4
      zk = 10
      call prinf('doing number of points = *',n,1)
c     quasi-periodic setup... (choose qpnei=0 for no QP; nei's can't touch!)
      qpnei = 0
      qpalpha = exp(ima*2*pi/7d0)
      qpdvec(1) = 2.7
      qpdvec(2) = 0
      call prinf('total # copies (incl neighbors) = *',1+2*qpnei,1)

c     parameters defining smooth star curve: aosc = amplitude of radial osc,
c     nosc = number of oscillations...
      aosc = 0.3
      nosc = 10

c     set up curve data at nodes for the radial function r(t), rp = dr/dt
      do i=1,n
         t = (2*pi*i)/n
         r = 1d0 + aosc*cos(nosc*t)
         rp = -nosc*aosc*sin(nosc*t)
         node(1,i) = r*cos(t)
         node(2,i) = r*sin(t)
         xp = -r*sin(t) + rp*cos(t)
         yp = r*cos(t) + rp*sin(t)
         sp(i) = sqrt(xp*xp + yp*yp)
         nvec(1,i) = yp / sp(i)
         nvec(2,i) = -xp / sp(i)
      enddo

c     set up a regular solution to Helmholtz equation in interior of curve...
      src(1) = 1d0
      src(2) = 2d0
      charge = (1,0)
c     use kernels to set  tau = -u, sig = u_n due to exterior monopole at src
      do i=1,n
         call fourkernels(src,snor,node(1,i),nvec(1,i), zk,0d0,0d0, f)
         tau(i) = -charge * f(1)
         sig(i) = charge * f(3)
      enddo

c     Apply operators on curve... (ifnd=0 so don't compute nd)
      iprec = 4
      alpertord = 16
      t1 = second()
      call lpevalselfcc(ier, iprec, n, node,nvec,sp, 1,sig,1,tau, zk,
     $     0d0, alpertord, 0,qpdvec,qpalpha,qpnei, 1,pot,0,nd)
      t2 = second()
      call prinf('done, ier = *',ier,1)
      call prin2('FMM+Alpert time = *',t2-t1,1)
      call prin2('points/sec counting all copies = *',
     $     (1+2*qpnei)*n/(t2-t1),1)
      call prin2('     points/sec = *',n/(t2-t1),1)
c     add interior jump relations
      do i=1,n
         pot(i) = pot(i) - tau(i)/2
c         nd(i) = nd(i) + sig(i)/2
      enddo
c      call prin2('pot=*',pot,10)
c      call prin2('tau=*',tau,10)


c     compute error of pot from u=-tau on the curve...
      errpot = 0
      do i=1,n
         errpot = errpot + sp(i)*abs(pot(i) - (-tau(i)))**2
      enddo
      errpot = sqrt(errpot/n)
c     note that neighbor copies produce zero exterior field by Calderon
      call prin2('L2-error in GRF potential on curve = *',errpot,1)
c      call prin2('pot = *',pot,n)
c      call prin2('tau = *',tau,n)
      return
      end

