% test lpevalselfcc interface & Green's Representation Theorem on closed curve
% Barnett 3/19/11, new QP interface & case added 3/24/11
clear
verb=0; N=1e4; k=10; qp.nei=0; alpertord=8; iprec=4;  % main params

a = 0.3; w = 11;  % smooth star-shaped curve: amplitude, number of wiggles
r = @(t) 1 + a*cos(w*t); rp = @(t) -a*w*sin(w*t); % radial func & its t-deriv
t = 2*pi*(1:N)/N;
nodes = [r(t).*cos(t); r(t).*sin(t)];
zp = [-r(t).*sin(t) + rp(t).*cos(t); r(t).*cos(t) + rp(t).*sin(t)]; % dz/dt
speed = sqrt(sum(zp.^2, 1)); nvec = [zp(2,:)./speed; -zp(1,:)./speed];

theta = pi/6; kvec = k*[cos(theta);sin(theta)]; % plane wave (int. Helm soln)
uc = exp(1i*kvec.'*nodes); unc = sum(nvec.*(1i*kvec*uc),1); % u, u_n on curve
sig = unc; tau = -uc;  % choose densities for interior GRF
ifsig = 1; iftau = 1; ifpot = 1; ifnd = 0;
qp.winding = 0; qp.dvec = [3.0;0.0]; qp.alpha = exp(2i*pi/7.0); % QP params
% Note that as long as no touching, QP copies don't contrib to field (Calderon)
if verb, figure; for j=-qp.nei:qp.nei  % show a pic of all sources
    plot(nodes(1,:)+j*qp.dvec(1), nodes(2,:)+j*qp.dvec(2),'.'); hold on; end
axis equal; drawnow; end
fprintf('applying S & D on curve with N=%d points...\n',N)
tic; U = lpevalselfcc(nodes,nvec,speed, ifsig,sig,iftau,tau, k,[],...
                      alpertord,iprec, qp, ifpot,ifnd); t=toc;
fprintf('FMM+Alpert done %.3g secs = %.3g pts (src) per sec\n', t, N/t);
fprintf('  counting QP copies, %.3g pts (src) per sec\n', N*(1+2*qp.nei)/t);
u = U.pot.' - tau/2;  % interior jump relation
fprintf('L2 err of pot on curve = %.3g\n', sqrt(sum(speed.*abs(u-uc).^2)/N))
